'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var pluginCommonjs = _interopDefault(require('@rollup/plugin-commonjs'));
var pluginJson = _interopDefault(require('@rollup/plugin-json'));
var pluginReplace = _interopDefault(require('@rollup/plugin-replace'));
var pluginNodeResolve = _interopDefault(require('@rollup/plugin-node-resolve'));
var rollup$1 = _interopDefault(require('rollup'));

const { rollup } = rollup$1;

/** @typedef { import("rollup").InputOptions } InputOptions */
/** @typedef { import("rollup").OutputOptions } OutputOptions */
/** @typedef { import("rollup").RollupOptions } RollupOptions */

/** @type { InputOptions } */
const ROLLUP_INPUT_DEFAULTS = {
  // Only bundle local package files
  external: (id, parent, isResolved) => {
    // Skip if already handled by plugin
    if (isResolved || parent.includes('?commonjs-proxy')) {
      return false;
    }
    return /^[^./\0]/.test(id);
  },
  plugins: [
    // @ts-ignore
    pluginReplace({
      'process.env.NODE_ENV': `"${process.env.NODE_ENV}"` || '"development"'
    }),
    // @ts-ignore
    pluginNodeResolve({
      mainFields: ['browser', 'module', 'main']
    }),
    // @ts-ignore
    pluginJson(),
    // @ts-ignore
    pluginCommonjs({
      sourceMap: false
    })
  ],
  treeshake: false
};
/** @type { OutputOptions } */
const ROLLUP_OUTPUT_DEFAULTS = {
  format: 'es',
  sourcemap: false
};

/**
 * Bundle package at 'id'
 *
 * @param { string } inputPath
 * @param { string } outputPath
 * @param { RollupOptions } [overrideOptions]
 * @param { (err?: Error) => void } fn
 * @returns { Promise<void> }
 */
var bundleWorker = async function bundle(
  inputPath,
  outputPath,
  overrideOptions,
  fn
) {
  const parsedOverrideOptions = parseOptions(overrideOptions);
  const inputOptions = {
    input: inputPath,
    ...ROLLUP_INPUT_DEFAULTS,
    ...parsedOverrideOptions.input
  };
  const outputOptions = {
    file: outputPath,
    ...ROLLUP_OUTPUT_DEFAULTS,
    ...parsedOverrideOptions.output
  };

  try {
    const bundled = await rollup(inputOptions);
    await bundled.write(outputOptions);
    fn();
  } catch (err) {
    console.log(err);
    fn(err);
  }
};

/**
 * Parse override options
 *
 * @param { RollupOptions } [options]
 * @returns { { input: object, output: object } }
 */
function parseOptions(options) {
  if (!options) {
    return { input: {}, output: {} };
  }

  const { input, treeshake, output = {}, watch, ...inputOverride } = options;
  // @ts-ignore
  const { file, format, sourcemap, ...outputOverride } = output;

  return { input: inputOverride, output: outputOverride };
}

module.exports = bundleWorker;
